#include <string.h>

#include "fitsimage.h"
#include "fvcontour.h"
#include "util.h"
#include "nan.h"

// It is a modified version of contour code found in Fv 2.4
// Fv may be obtained from the HEASARC (High Energy Astrophysics Science
// Archive Research Center) FTOOLS Web site at:
// http://heasarc.gsfc.nasa.gov/ftools/fv.html

// The original author is unknown.

FVContourPlot::FVContourPlot(Frame* p, FitsImage* fits, const char* c,
			     int cnt, float r, float l, float h,
			     Frame::ColorScaleType scaleType)
  : resolution(r), ContourPlot(p, c)
{
  switch (scaleType) {
  case Frame::LINEARSCALE:
    scale = new LinearContourScale(cnt, l, h);
    break;
  case Frame::LOGSCALE:
    scale = new LogContourScale(cnt, l, h);
    break;
  case Frame::SQUAREDSCALE:
    scale = new SquaredContourScale(cnt, l, h);
    break;
  case Frame::SQRTSCALE:
    scale = new SqrtContourScale(cnt, l, h);
    break;
  }

  if (resolution == 1.)
    unity(fits);
  else
    block(fits);
}

FVContourPlot::FVContourPlot(Frame* p, FitsImage* fits, const char* c,
			     int cnt, float r, float* levels)
  : resolution(r), ContourPlot(p, c)
{
  scale = new ContourScale(cnt, levels);

  if (resolution == 1.)
    unity(fits);
  else
    block(fits);
}

FVContourPlot::~FVContourPlot()
{
  if (scale)
    delete scale;
}

void FVContourPlot::append(FitsImage* fits)
{
  if (resolution == 1.)
    unity(fits);
  else
    block(fits);
}

void FVContourPlot::unity(FitsImage* fits)
{
  int width = fits->width();
  int height = fits->height();

  float* img = new float[width * height];
  for(int j=0; j<height; j++)
    for(int i=0; i<width; i++) {
      int k = j*width + i;
      img[k] = fits->getValueFloat(k);
    }

  int status = build(width, height, img, fits->getDataToRef());
  delete [] img;

  if (status)
    cerr << "FVContour error" << endl;
}

void FVContourPlot::block(FitsImage* fits)
{
  int width = fits->width();
  int height = fits->height();

  int w2 = (int)(width/resolution);
  int h2 = (int)(height/resolution);

  Matrix m = 
    Translate(-width/2., -height/2.) * 
    Scale(1./resolution) * 
    Translate(w2/2., h2/2.);
  Matrix n = m.invert();

  float* img = new float[w2 * h2];
  short* count = new short[w2 * h2];

  memset(img, 0, w2*h2*sizeof(float));
  memset(count, 0, w2*h2*sizeof(short));

  register double m00 = m.matrix(0,0);
  register double m10 = m.matrix(1,0);
  register double m20 = m.matrix(2,0);
  register double m01 = m.matrix(0,1);
  register double m11 = m.matrix(1,1);
  register double m21 = m.matrix(2,1);

  for (int j=0; j<height; j++)
    for (int i=0; i<width; i++) {

      register double srcX = i*m00 + j*m10 + m20;
      register double srcY = i*m01 + j*m11 + m21;

      if (srcX >= 0 && srcX < w2 && srcY >= 0 && srcY < h2) {
	int k = ((int)srcY)*w2 + (int)srcX;
	img[k] += fits->getValueFloat(((int)j)*width + (int)i);
	count[k]++;
      }
    }

  for (int k=0; k<w2*h2; k++)
    if (count[k])
      img[k] /= count[k];

  Matrix w = n * fits->getDataToRef();
  int status = build(w2, h2, img, w);

  delete [] img;
  delete [] count;

  if (status)
    cerr << "FVContour error" << endl;
}

int FVContourPlot::build(int xdim, int ydim, float *image, Matrix& matrix)
{
  int status = 0;

  long nelem = xdim*ydim;
  char* usedGrid = new char[nelem];
  if (!usedGrid)
    return 1;

  float** rows = new float*[ydim];
  if (!rows)
    return 1;

  for (int j=0; j<ydim; j++)
    rows[j] = image + j*xdim;

  for (int c=0; c<scale->size() && !status; c++) {
    float cntour = scale->level(c);
    for (long elem=0; elem<nelem; elem++)
      usedGrid[elem] = 0;

    //  Search outer edge

    int i,j;

    //  Search top

    for (j=0, i=0; i<xdim-1 && !status; i++)
      if (rows[j][i]<cntour && cntour<=rows[j][i+1])
	status = trace(xdim, ydim, cntour, i, j, top, rows, usedGrid, matrix);

    //  Search right

    for (j=0; j<ydim-1 && !status; j++)
      if (rows[j][i]<cntour && cntour<=rows[j+1][i])
	status = trace(xdim, ydim, cntour, i-1, j, right, rows, usedGrid, 
		       matrix);

    //  Search Bottom

    for (i--; i>=0 && !status; i--)
      if (rows[j][i+1]<cntour && cntour<=rows[j][i])
	status = trace(xdim, ydim, cntour, i, j-1, bottom, rows, usedGrid, 
		       matrix);

    //  Search Left

    for (i=0, j--; j>=0 && !status; j--)
      if (rows[j+1][i]<cntour && cntour<=rows[j][i])
	status = trace(xdim, ydim, cntour, i, j, left, rows, usedGrid, matrix);

    //  Search each row of the image

    for (j=1; j<ydim-1 && !status; j++)
      for (i=0; i<xdim-1 && !status; i++)
	if (!usedGrid[j*xdim + i] && rows[j][i]<cntour && cntour<=rows[j][i+1])
	  status = trace(xdim, ydim, cntour, i, j, top, rows, usedGrid, 
			 matrix);
  }

  delete [] usedGrid;
  delete [] rows;

  return status;
}

int FVContourPlot::trace(int xdim, int ydim, float cntr,
				int xCell, int yCell, int side, 
				float** rows, char* usedGrid, Matrix& matrix)
{
  int i = xCell;
  int j = yCell;
  int origSide = side;

  int init = 1;
  int done = (i<0 || i>=xdim-1 || j<0 && j>=ydim-1);

  while (!done) {
    int flag = 0;
    float a = rows[j][i];
    float b = rows[j][i+1];
    float c = rows[j+1][i+1];
    float d = rows[j+1][i];

    float X, Y;
    if (init) {
      init = 0;
      switch (side) {
      case top:
	X = (cntr-a) / (b-a) + i;
	Y = j;
	break;
      case right:
	X = i+1;
	Y = (cntr-b) / (c-b) + j;
	break;
      case bottom:
	X = (cntr-c) / (d-c) + i;
	Y = j+1;
	break;
      case left:
	X = i;
	Y = (cntr-a) / (d-a) + j;
	break;
      }

    }
    else {
      if (side==top)
	usedGrid[j*xdim + i] = 1;

      do {
	if (++side == none)
	  side = top;

	switch (side) {
	case top:
	  if (a>=cntr && cntr>b) {
	    flag = 1;
	    X = (cntr-a) / (b-a) + i;
	    Y = j;
	    j--;
	  }
	  break;
	case right:
	  if( b>=cntr && cntr>c ) {
	    flag = 1;
	    X = i+1;
	    Y = (cntr-b) / (c-b) + j;
	    i++;
	  }
	  break;
	case bottom:
	  if( c>=cntr && cntr>d ) {
	    flag = 1;
	    X = (cntr-d) / (c-d) + i;
	    Y = j+1;
	    j++;
	  }
	  break;
	case left:
	  if( d>=cntr && cntr>a ) {
	    flag = 1;
	    X = i;
	    Y = (cntr-a) / (d-a) + j;
	    i--;
	  }
	  break;
	}
      } while (!flag);

      if (++side == none)
	side = top;
      if (++side == none)
	side = top;
      if (i==xCell && j==yCell && side==origSide)
	done = 1;
      if (i<0 || i>=xdim-1 || j<0 || j>=ydim-1)
	done = 1;
    }

    contours->append(new Vertex(Vector(X+.5,Y+.5)*matrix));

    if (done)
      contours->append(new Vertex(DBL_MAX, DBL_MAX));
  }

  return 0;
}
